@@ -143,8 +143,9 @@ def _get_other_converter(parser, config):
 
 def uxf_to_csv(infile, outfile, *, verbose=True, replace_imports=False,
                drop_unused=False, indent='  ', wrap_width=96):
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo = uxf.load(infile, on_error=on_error, drop_unused=drop_unused,
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
+    uxo = uxf.load(infile, on_event=on_event, drop_unused=drop_unused,
                    replace_imports=replace_imports)
     value = uxo.value
     if isinstance(value, uxf.Table):
@@ -169,10 +170,11 @@ def uxf_to_csv(infile, outfile, *, verbose=True, replace_imports=False,
 def csv_to_uxf(infile, outfile, *, fieldnames=False, verbose=True,
                replace_imports=False, drop_unused=False, indent='  ',
                wrap_width=96):
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
     value, filename, tclasses = _read_csv_to_data(infile, fieldnames)
     uxf.dump(outfile, uxf.Uxf(value, custom=filename, tclasses=tclasses),
-             on_error=on_error,
+             on_event=on_event,
              format=uxf.Format(indent=indent, wrap_width=wrap_width))
 
 
@@ -205,17 +207,19 @@ def multi_csv_to_uxf(infiles, outfile, *, fieldnames=False, verbose=True,
         data.append(datum)
         if new_tclasses:
             tclasses.update(new_tclasses)
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
     uxf.dump(outfile,
              uxf.Uxf(data, custom=' '.join(infiles), tclasses=tclasses),
-             on_error=on_error,
+             on_event=on_event,
              format=uxf.Format(indent=indent, wrap_width=wrap_width))
 
 
 def uxf_to_json(infile, outfile, *, verbose=True, replace_imports=False,
                 drop_unused=False, indent='  ', wrap_width=96):
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo = uxf.load(infile, on_error=on_error, drop_unused=drop_unused,
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
+    uxo = uxf.load(infile, on_event=on_event, drop_unused=drop_unused,
                    replace_imports=replace_imports)
     d = {}
     if uxo.custom is not None:
@@ -314,10 +318,11 @@ def json_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
         for tclass in tclass_list:
             tclasses[tclass.ttype] = tclass
     data = d.get(JSON_DATA)
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
     uxo = uxf.Uxf(data, custom=custom, tclasses=tclasses, comment=comment)
     uxo.imports = imports
-    uxf.dump(outfile, uxo, on_error=on_error,
+    uxf.dump(outfile, uxo, on_event=on_event,
              format=uxf.Format(indent=indent, wrap_width=wrap_width))
 
 
@@ -365,7 +370,8 @@ def _json_naturalize(d):
 
 def ini_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
                drop_unused=False, indent='  ', wrap_width=96):
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
     ini = configparser.ConfigParser()
     ini.read(infile)
     data = uxf.Map()
@@ -375,14 +381,15 @@ def ini_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
             m = data[section] = uxf.Map()
             for key, value in d.items():
                 m[uxf.naturalize(key)] = uxf.naturalize(value)
-    uxf.dump(outfile, uxf.Uxf(data, custom=infile), on_error=on_error,
+    uxf.dump(outfile, uxf.Uxf(data, custom=infile), on_event=on_event,
              format=uxf.Format(indent=indent, wrap_width=wrap_width))
 
 
 def uxf_to_sqlite(infile, outfile, *, verbose=True, replace_imports=False,
                   drop_unused=False, indent='  ', wrap_width=96):
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo = uxf.load(infile, on_error=on_error, drop_unused=drop_unused,
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
+    uxo = uxf.load(infile, on_event=on_event, drop_unused=drop_unused,
                    replace_imports=replace_imports)
     if isinstance(uxo.value, uxf.Table):
         _uxf_to_sqlite(outfile, [uxo.value])
@@ -445,8 +452,9 @@ def _populate_table(db, table, ttype):
 def sqlite_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
                   drop_unused=False, indent='  ', wrap_width=96):
     uxo = _sqlite_to_uxf(infile)
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo.dump(outfile, on_error=on_error,
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
+    uxo.dump(outfile, on_event=on_event,
              format=uxf.Format(indent=indent, wrap_width=wrap_width))
 
 
@@ -485,8 +493,9 @@ def _sqlite_to_uxf(infile):
 
 def uxf_to_xml(infile, outfile, *, verbose=True, replace_imports=False,
                drop_unused=False, indent='  ', wrap_width=96):
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo = uxf.load(infile, on_error=on_error, drop_unused=drop_unused,
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
+    uxo = uxf.load(infile, on_event=on_event, drop_unused=drop_unused,
                    replace_imports=replace_imports)
     _uxf_to_xml(uxo, outfile, indent=indent)
 
@@ -630,9 +639,10 @@ def _xml_add_scalar(tree, root, value):
 
 def xml_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
                drop_unused=False, indent='  ', wrap_width=96):
-    on_error = functools.partial(uxf.on_error, verbose=verbose)
+    on_event = functools.partial(uxf.on_event, verbose=verbose,
+                                 prefix='uxfconvert')
     uxo = _xml_to_uxf(infile)
-    uxo.dump(outfile, on_error=on_error,
+    uxo.dump(outfile, on_event=on_event,
              format=uxf.Format(indent=indent, wrap_width=wrap_width))
 
 
@@ -668,9 +678,10 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
         elif name == 'import':
             self.imports_list.append(d['filename'])
         elif name == 'ttype':
-            self.tclass = uxf.TClass(d['name'], comment=d.get('comment'))
+            self.tclass = uxf.TClassBuilder(d['name'],
+                                            comment=d.get('comment'))
         elif name == 'field':
-            self.tclass.append(d['name'], d.get('vtype'))
+            self.tclass.append(uxf.Field(d['name'], d.get('vtype')))
         elif name == 'map':
             container = uxf.Map(ktype=d.get('ktype'), vtype=d.get('vtype'),
                                 comment=d.get('comment'))
@@ -680,9 +691,8 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
                                  comment=d.get('comment'))
             self.start_container(container)
         elif name == 'table':
-            container = uxf.Table(uxf.TClass(d['name']),
-                                  comment=d.get('comment'))
-            container.tclass = self.uxo.tclasses[d['name']]
+            tclass = self.uxo.tclasses[d['name']]
+            container = uxf.Table(tclass, comment=d.get('comment'))
             self.start_container(container)
         elif name in {'key', 'value'}:
             pass # container.append() doesn't need this distinction
@@ -716,7 +726,7 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
             if self.imports_list:
                 self.uxo.imports = _get_imports(self.imports_list)
         elif name == 'ttype':
-            self.uxo.tclasses[self.tclass.ttype] = self.tclass
+            self.uxo.tclasses[self.tclass.ttype] = self.tclass.build()
             self.tclass = None
         elif name in {'map', 'list', 'table'}:
             self.stack.pop()
